@@ -18,12 +18,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <libglom/libglom_config.h> // For GLOM_ENABLE_SQLITE, GLOM_ENABLE_CLIENT_ONLY
+#include "config.h" //For GLOM_ENABLE_SQLITE
+#include <libglom/libglom_config.h> // For GLOM_ENABLE_CLIENT_ONLY
 
 #include <glom/frame_glom.h>
 #include <glom/application.h>
-#include <glom/dialog_import_csv.h>
-#include <glom/dialog_import_csv_progress.h>
+#include <glom/import_csv/dialog_import_csv.h>
+#include <glom/import_csv/dialog_import_csv_progress.h>
 #include <libglom/appstate.h>
 
 #include <libglom/connectionpool.h>
@@ -38,14 +39,14 @@
 #endif
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-#include "mode_design/users/dialog_groups_list.h"
-#include "dialog_database_preferences.h"
-#include "reports/dialog_layout_report.h"
+#include <glom/mode_design/users/dialog_groups_list.h>
+#include <glom/mode_design/dialog_database_preferences.h>
+#include <glom/mode_design/report_layout/dialog_layout_report.h>
 #include <glom/mode_design/print_layouts/window_print_layout_edit.h>
 #include <glom/mode_design/dialog_add_related_table.h>
 #include <glom/mode_design/script_library/dialog_script_library.h>
-#include <glom/dialog_new_self_hosted_connection.h>
-#include "relationships_overview/dialog_relationships_overview.h"
+#include <glom/mode_design/dialog_initial_password.h>
+#include <glom/mode_design/relationships_overview/dialog_relationships_overview.h>
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
 #include <glom/utils_ui.h>
@@ -54,7 +55,7 @@
 #include <libglom/data_structure/layout/report_parts/layoutitem_summary.h>
 #include <libglom/data_structure/layout/report_parts/layoutitem_fieldsummary.h>
 
-#include <glom/reports/report_builder.h>
+#include <glom/report_builder.h>
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 #include <glom/mode_design/dialog_add_related_table.h>
 #include <glom/mode_design/script_library/dialog_script_library.h>
@@ -63,9 +64,12 @@
 
 #ifdef GLOM_ENABLE_MAEMO
 #include <hildonmm/note.h>
+#include <hildonmm/entry.h>
+#include <hildonmm/text-view.h>
+#include <hildonmm/button.h>
 #endif
 
-#include "filechooser_export.h"
+#include <glom/filechooser_export.h>
 #include <glom/glom_privs.h>
 #include <sstream> //For stringstream.
 #include <fstream>
@@ -77,19 +81,22 @@ namespace Glom
 Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
 : PlaceHolder(cobject, builder),
   m_pLabel_Table(0),
+  m_box_header(0),
   m_box_footer(0),
   m_pLabel_Mode(0),
   m_pLabel_userlevel(0),
-  m_pBox_QuickFind(0),
-  m_pEntry_QuickFind(0),
-  m_pButton_QuickFind(0),
   m_pBox_RecordsCount(0),
   m_pLabel_RecordsCount(0),
   m_pLabel_FoundCount(0),
   m_pButton_FindAll(0),
   m_pBox_Mode(0),
+#ifndef GLOM_ENABLE_MAEMO
   m_pBox_Tables(0),
   m_pDialog_Tables(0),
+#endif //GLOM_ENABLE_MAEMO
+  m_pBox_QuickFind(0),
+  m_pEntry_QuickFind(0),
+  m_pButton_QuickFind(0),
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   m_pDialog_Reports(0),
   m_pDialogLayoutReport(0),
@@ -101,36 +108,62 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>
   m_pDialog_Relationships(0),
   m_dialog_addrelatedtable(0),
   m_dialog_relationships_overview(0),
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-  m_pDialogConnection(0),
   m_dialog_progess_connection_initialize(0),
+#endif // !GLOM_ENABLE_CLIENT_ONLY
   m_dialog_progess_connection_startup(0),
-  m_dialog_progess_connection_cleanup(0)
+  m_dialog_progess_connection_cleanup(0),
+  m_pDialogConnection(0)
 {
   //Load widgets from glade file:
   builder->get_widget("label_table_name", m_pLabel_Table);
 
+  builder->get_widget("hbox_header", m_box_header);
   builder->get_widget("hbox_footer", m_box_footer);
   builder->get_widget("label_mode", m_pLabel_Mode);
   builder->get_widget("label_user_level", m_pLabel_userlevel);
   
-  //Hide the footer on maemo (It takes too much space),
+  //Hide unnecessary widgets on maemo that take too much space,
   //and reduce the border width:
   #ifdef GLOM_ENABLE_MAEMO
+  m_box_header->hide();
   m_box_footer->hide();
   set_border_width(Glom::Utils::DEFAULT_SPACING_LARGE);
   #endif
 
-  builder->get_widget("hbox_quickfind", m_pBox_QuickFind);
-  m_pBox_QuickFind->hide();
-
-  builder->get_widget("entry_quickfind", m_pEntry_QuickFind);
+  //QuickFind widgets:
+  //We don't use Glade for these, so it easier to modify them for the Maemo port.
+  m_pBox_QuickFind = Gtk::manage(new Gtk::HBox(false, Utils::DEFAULT_SPACING_SMALL));
+  Gtk::Label* label = Gtk::manage(new Gtk::Label(_("Quick Find")));
+  m_pBox_QuickFind->pack_start(*label, Gtk::PACK_SHRINK);
+  
+  #ifndef GLOM_ENABLE_MAEMO
+  m_pEntry_QuickFind = Gtk::manage(new Gtk::Entry());
+  #else
+  m_pEntry_QuickFind = Gtk::manage(new Hildon::Entry(Gtk::Hildon::SIZE_AUTO));
+  #endif
   m_pEntry_QuickFind->signal_activate().connect(
    sigc::mem_fun(*this, &Frame_Glom::on_button_quickfind) ); //Pressing Enter here is like pressing Find.
 
-  builder->get_widget("button_quickfind", m_pButton_QuickFind);
+  m_pBox_QuickFind->pack_start(*m_pEntry_QuickFind, Gtk::PACK_EXPAND_WIDGET);
+  #ifndef GLOM_ENABLE_MAEMO
+  m_pButton_QuickFind = Gtk::manage(new Gtk::Button(_("_Find"), true));
+  #else
+  m_pButton_QuickFind = Gtk::manage(new Hildon::Button(Gtk::Hildon::SIZE_AUTO, 
+    Hildon::BUTTON_ARRANGEMENT_VERTICAL, _("Find"), _("Search for records")));
+  #endif
   m_pButton_QuickFind->signal_clicked().connect(
     sigc::mem_fun(*this, &Frame_Glom::on_button_quickfind) );
+  m_pBox_QuickFind->pack_start(*m_pButton_QuickFind, Gtk::PACK_SHRINK);
+  
+  m_pBox_QuickFind->show_all_children();
+  m_pBox_QuickFind->hide();
+
+  #ifndef GLOM_ENABLE_MAEMO
+  PlaceHolder* placeholder_quickfind = 0;
+  builder->get_widget_derived("vbox_quickfind", placeholder_quickfind);
+  placeholder_quickfind->add(*m_pBox_QuickFind);
+  #endif //GLOM_ENABLE_MAEMO
+
 
   builder->get_widget("hbox_records_count", m_pBox_RecordsCount);
   builder->get_widget("label_records_count", m_pLabel_RecordsCount);
@@ -147,7 +180,6 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>
 
   m_Notebook_Find.signal_find_criteria.connect(sigc::mem_fun(*this, &Frame_Glom::on_notebook_find_criteria));
   m_Notebook_Find.show();
-
   m_Notebook_Data.signal_record_details_requested().connect(sigc::mem_fun(*this, &Frame_Glom::on_notebook_data_record_details_requested));
   m_Notebook_Data.signal_switch_page().connect(sigc::mem_fun(*this, &Frame_Glom::on_notebook_data_switch_page));
   m_Notebook_Data.show();
@@ -158,21 +190,34 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>
   add_view(&m_Notebook_Find); //Also a composite view.
 
   on_userlevel_changed(AppState::USERLEVEL_OPERATOR); //A default to show before a document is created or loaded.
+  
+  #ifdef GLOM_ENABLE_MAEMO
+  m_maemo_window_find.set_title(_("Glom: Find"));
+  
+  Gtk::VBox* vbox = Gtk::manage(new Gtk::VBox(false, Utils::DEFAULT_SPACING_SMALL));
+  vbox->pack_start(*m_pBox_QuickFind, Gtk::PACK_SHRINK);
+  vbox->pack_start(m_Notebook_Find, Gtk::PACK_EXPAND_WIDGET);
+  m_maemo_window_find.add(*vbox);
+  m_maemo_window_find.show_all_children();
+  #endif //GLOM_ENABLE_MAEMO
 }
 
 Frame_Glom::~Frame_Glom()
 {
+#ifndef GLOM_ENABLE_MAEMO
   if(m_pBox_Tables)
     remove_view(m_pBox_Tables);
 
-  remove_view(&m_Notebook_Data); //Also a composite view.
-  remove_view(&m_Notebook_Find); //Also a composite view.
-
   if(m_pDialog_Tables)
   {
     delete m_pDialog_Tables;
     m_pDialog_Tables = 0;
   }
+#endif //GLOM_ENABLE_MAEMO
+
+  remove_view(&m_Notebook_Data); //Also a composite view.
+  remove_view(&m_Notebook_Find); //Also a composite view.
+
 
   if(m_pDialogConnection)
   {
@@ -181,11 +226,6 @@ Frame_Glom::~Frame_Glom()
     m_pDialogConnection = 0;
   }
   
-  if(m_dialog_progess_connection_initialize)
-  {
-    delete m_dialog_progess_connection_initialize;
-    m_dialog_progess_connection_initialize = 0;
-  }
   
   if(m_dialog_progess_connection_startup)
   {
@@ -200,6 +240,12 @@ Frame_Glom::~Frame_Glom()
   }
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
+  if(m_dialog_progess_connection_initialize)
+  {
+    delete m_dialog_progess_connection_initialize;
+    m_dialog_progess_connection_initialize = 0;
+  }
+
   if(m_pBox_Reports)
     remove_view(m_pBox_Reports);
 
@@ -263,14 +309,27 @@ void Frame_Glom::set_databases_selected(const Glib::ustring& strName)
 
 void Frame_Glom::on_box_tables_selected(const Glib::ustring& strName)
 {
+#ifndef GLOM_ENABLE_MAEMO
   if(m_pDialog_Tables)
     m_pDialog_Tables->hide();
+#endif //GLOM_ENABLE_MAEMO
 
   show_table(strName);
 }
 
 void Frame_Glom::set_mode_widget(Gtk::Widget& widget)
 {
+  #ifdef GLOM_ENABLE_MAEMO
+  //On Maemo, the find UI is always shown in a separate window instead.
+  if(&widget == &m_Notebook_Find)
+    return;
+  else
+  {
+    //Make sure that this is hidden:
+    m_maemo_window_find.hide();
+  }
+  #endif
+
   //Remove current contents.
   //I wish that there was a better way to do this:
   //Trying to remove all of them leads to warnings,
@@ -279,7 +338,7 @@ void Frame_Glom::set_mode_widget(Gtk::Widget& widget)
   App_Glom* pApp = dynamic_cast<App_Glom*>(get_app_window());
   if(pApp)
   {
-    Glib::RefPtr<Gtk::UIManager> ui_manager = pApp->get_ui_manager();
+    //Glib::RefPtr<Gtk::UIManager> ui_manager = pApp->get_ui_manager();
 
     Notebook_Glom* notebook_current = dynamic_cast<Notebook_Glom*>(m_pBox_Mode->get_child());
     if(notebook_current)
@@ -428,7 +487,7 @@ void Frame_Glom::show_table_allow_empty(const Glib::ustring& table_name, const G
     case(MODE_Find):
     {
       strMode = _("Find");
-      m_Notebook_Find.init_db_details(m_table_name, get_document()->get_active_layout_platform());
+      m_Notebook_Find.init_db_details(m_table_name, get_active_layout_platform(get_document()));
       set_mode_widget(m_Notebook_Find);
       break;
     }
@@ -567,7 +626,7 @@ void Frame_Glom::on_menu_file_export()
   if(!document)
     return;
 
-  Document::type_list_layout_groups mapGroupSequence = document->get_data_layout_groups_plus_new_fields("details", m_table_name, document->get_active_layout_platform());
+  Document::type_list_layout_groups mapGroupSequence = document->get_data_layout_groups_plus_new_fields("details", m_table_name, get_active_layout_platform(document));
 
   Gtk::Window* pWindowApp = get_app_window();
   g_assert(pWindowApp);
@@ -582,6 +641,8 @@ void Frame_Glom::on_menu_file_export()
 
   //Ask the user for the new file location, and to optionally modify the format:
   FileChooser_Export dialog;
+  dialog.set_transient_for(*get_app_window());
+  dialog.set_do_overwrite_confirmation();
   dialog.set_export_layout(mapGroupSequence, m_table_name, get_document());
   const int response = dialog.run();
   dialog.hide();
@@ -589,10 +650,12 @@ void Frame_Glom::on_menu_file_export()
   if((response == Gtk::RESPONSE_CANCEL) || (response == Gtk::RESPONSE_DELETE_EVENT))
     return;
 
-  const std::string filepath = dialog.get_filename();
+  std::string filepath = dialog.get_filename();
   if(filepath.empty())
     return;
 
+  filepath = Utils::get_filepath_with_extension(filepath, "csv");
+
   dialog.get_layout_groups(mapGroupSequence);
   //std::cout << "DEBUG 0: mapGroupSequence.size()=" << mapGroupSequence.size() << std::endl;
 
@@ -696,7 +759,7 @@ void Frame_Glom::export_data_to_string(Glib::ustring& the_string, const FoundSet
           //if(layout_item->m_field.get_glom_type() != Field::TYPE_IMAGE) //This is too much data.
           //{
             if(!row_string.empty())
-              row_string += ",";
+              row_string += ',';
 
             //Output data in canonical SQL format, ignoring the user's locale, and ignoring the layout formatting:
             row_string += layout_item->get_full_field_details()->to_file_format(value);
@@ -751,13 +814,46 @@ void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const FoundSet&
               row_string += ",";
 
             //Output data in canonical SQL format, ignoring the user's locale, and ignoring the layout formatting:
-            row_string += layout_item->get_full_field_details()->to_file_format(value);
+            sharedptr<const Field> field = layout_item->get_full_field_details();
+            if(!field)
+            {
+              std::cerr << "Glom: Frame_Glom::export_data_to_stream(): A field was null." << std::endl;
+              return;
+            }
+
+            const Glib::ustring field_text = field->to_file_format(value);
 
             if(layout_item->get_glom_type() == Field::TYPE_IMAGE) //This is too much data.
             {
-              if(!Conversions::value_is_empty(value))
-                std::cout << "  field name=" << layout_item->get_name() << ", image value not empty=" << std::endl;
+              // Some extra debug checks, 
+              // though we believe that all these problems are now fixed in File::to_file_format():
+
+              const char* newline_to_find = "\r\n";
+              size_t pos = field_text.find_first_of(newline_to_find);
+              if(pos != std::string::npos)
+              {
+                std::cerr << "export: binary data field text contains an unexpected newline: " << field_text << std::endl;
+                continue;
+              }
+
+              const char* quote_to_find = "\"";
+              pos = field_text.find_first_of(quote_to_find);
+              if(pos != std::string::npos)
+              {
+                std::cerr << "export: binary data field text contains an unexpected quote: " << field_text << std::endl;
+                continue;
+              }
             }
+            
+            if(layout_item->get_glom_type() == Field::TYPE_TEXT)
+            {
+              //The CSV RFC says text may be quoted and should be if it has newlines:
+              row_string += ("\"" + field_text + "\""); 
+            }
+            else
+              row_string += field_text;
+
+           
             //std::cout << "  field name=" << layout_item->get_name() << ", value=" << layout_item->m_field.sql(value) << std::endl;
           //}
         }
@@ -772,8 +868,7 @@ void Frame_Glom::on_menu_file_import()
 {
   if(m_table_name.empty())
   {
-    Gtk::MessageDialog dialog(*get_app_window(), "There is no table to import data into", false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK);
-    dialog.run();
+    Utils::show_ok_dialog(_("No Table"), _("There is no table in to which data could be imported."), *get_app_window(), Gtk::MESSAGE_ERROR);
   }
   else
   {
@@ -793,8 +888,16 @@ void Frame_Glom::on_menu_file_import()
     {
       file_chooser.hide();
 
+      //GtkBuilder can't find top-level objects (GtkAdjustments in this case),
+      //that one top-level object references.
+      //See http://bugzilla.gnome.org/show_bug.cgi?id=575714
+      //so we need to this silliness. murrayc.
+      std::list<Glib::ustring> builder_ids;
+      builder_ids.push_back("dialog_import_csv");
+      builder_ids.push_back("adjustment1");
+
       Dialog_Import_CSV* dialog = 0;
-      Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom.glade"), "dialog_import_csv");
+      Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom.glade"), builder_ids);
       refXml->get_widget_derived("dialog_import_csv", dialog);
       add_view(dialog);
 
@@ -804,7 +907,16 @@ void Frame_Glom::on_menu_file_import()
         dialog->hide();
 
         Dialog_Import_CSV_Progress* progress_dialog = 0;
-        Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom.glade"), "dialog_import_csv_progress");
+
+        //GtkBuilder can't find top-level objects (GtkTextBuffer in this case),
+        //that one top-level object references.
+        //See http://bugzilla.gnome.org/show_bug.cgi?id=575714
+        //so we need to this silliness. murrayc.
+        std::list<Glib::ustring> builder_ids;
+        builder_ids.push_back("dialog_import_csv_progress");
+        builder_ids.push_back("textbuffer1");
+
+        Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom.glade"), builder_ids);
         refXml->get_widget_derived("dialog_import_csv_progress", progress_dialog);
         add_view(progress_dialog);
 
@@ -830,7 +942,6 @@ void Frame_Glom::on_menu_file_import()
     }
   }
 }
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 void Frame_Glom::on_menu_file_toggle_share(const Glib::RefPtr<Gtk::ToggleAction>& action)
 {
@@ -1037,6 +1148,7 @@ void Frame_Glom::on_menu_file_toggle_share(const Glib::RefPtr<Gtk::ToggleAction>
     pApp->update_network_shared_ui();
   }
 }
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 void Frame_Glom::on_menu_file_print()
 {
@@ -1080,18 +1192,38 @@ void Frame_Glom::on_menu_Mode_Find()
   if(previously_in_data_mode && (list_or_details == Notebook_Data::DATA_VIEW_Details))
     m_Notebook_Data.set_current_view(Notebook_Data::DATA_VIEW_List);
 
-  if(set_mode(MODE_Find))
-  {
-    show_table(m_table_name);
+  if(!set_mode(MODE_Find))
+    return;
+  
+  show_table(m_table_name);
 
-    if(previously_in_data_mode)
-    {
-      //Show the same layout in Find mode as was just being viewed in Data mode:
-      m_Notebook_Find.set_current_view(list_or_details);
-    }
+  if(previously_in_data_mode)
+  {
+    //Show the same layout in Find mode as was just being viewed in Data mode:
+    m_Notebook_Find.set_current_view(list_or_details);
   }
+  
+  #ifdef GLOM_ENABLE_CLIENT_ONLY
+  Gtk::Window* parent = get_app_window();
+  g_assert(parent);
+  if(parent)
+    m_maemo_window_find.set_transient_for(*parent);
+    
+  m_maemo_window_find.show(); //TODO: Switch back to data on hide?
+  #endif
 }
 
+#ifdef GLOM_ENABLE_MAEMO
+void Frame_Glom::on_menu_add_record()
+{
+  BusyCursor busy_cursor(get_app_window());
+  
+  //Note: This should only be called in Data mode.
+  m_Notebook_Data.do_menu_file_add_record();
+}
+#endif //GLOM_ENABLE_MAEMO
+
+
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 void Frame_Glom::on_menu_Reports_EditReports()
 {
@@ -1259,18 +1391,18 @@ void Frame_Glom::do_menu_Navigate_Table(bool open_default)
   if(open_default)
     default_table_name = get_document()->get_default_table();
   
+#ifndef GLOM_ENABLE_MAEMO
   //Create the dialog, if it has not already been created:
   if(!m_pBox_Tables)
   {
     Utils::get_glade_widget_derived_with_warning("box_navigation_tables", m_pBox_Tables);
-    m_pDialog_Tables = new Dialog_Glom(m_pBox_Tables);
+    m_pDialog_Tables = new Window_BoxHolder(m_pBox_Tables, _("Tables"));
     m_pDialog_Tables->signal_hide().connect(sigc::mem_fun(*this, &Frame_Glom::on_dialog_tables_hide));
 
     Gtk::Window* pWindow = get_app_window();
     if(pWindow)
       m_pDialog_Tables->set_transient_for(*pWindow);
 
-    m_pDialog_Tables->get_vbox()->pack_start(*m_pBox_Tables);
     m_pDialog_Tables->set_default_size(300, 400);
     m_pBox_Tables->show_all();
     add_view(m_pBox_Tables);
@@ -1283,6 +1415,7 @@ void Frame_Glom::do_menu_Navigate_Table(bool open_default)
     BusyCursor busy_cursor(get_app_window());
     m_pBox_Tables->init_db_details();
   }
+  #endif // !GLOM_ENABLE_CLIENT_ONLY
 
   //Let the user choose a table:
   //m_pDialog_Tables->set_policy(false, true, false); //TODO_port
@@ -1294,8 +1427,13 @@ void Frame_Glom::do_menu_Navigate_Table(bool open_default)
   }
   else
   {
+    #ifndef GLOM_ENABLE_CLIENT_ONLY
     m_pDialog_Tables->show();
+    #else
+    //For Maemo: TODO
+    #endif // !GLOM_ENABLE_CLIENT_ONLY
   }
+
 }
 
 const Gtk::Window* Frame_Glom::get_app_window() const
@@ -1414,30 +1552,32 @@ void Frame_Glom::on_userlevel_changed(AppState::userlevels userlevel)
 
 void Frame_Glom::show_table_title()
 {
-  if(get_document())
+  Document* document = dynamic_cast<Document*>(get_document());
+  if(!document)
+    return;
+
+  //Show the table title:
+  Glib::ustring table_label = document->get_table_title(m_table_name);
+  if(!table_label.empty())
   {
-    //Show the table title:
-    Glib::ustring table_label = get_document()->get_table_title(m_table_name);
-    if(!table_label.empty())
-    {
-      Document* document = dynamic_cast<Document*>(get_document());
-      if(document)
-      {
-        if(document->get_userlevel() == AppState::USERLEVEL_DEVELOPER)
-          table_label += " (" + m_table_name + ")"; //Show the table name as well, if in developer mode.
-      }
-    }
-    else //Use the table name if there is no table title.
-      table_label = m_table_name;
+    if(document->get_userlevel() == AppState::USERLEVEL_DEVELOPER)
+      table_label += " (" + m_table_name + ")"; //Show the table name as well, if in developer mode.
+  }
+  else //Use the table name if there is no table title.
+    table_label = m_table_name;
 
 #ifdef GLOM_ENABLE_MAEMO
-    // xx-large is too large on maemo, taking away too much (vertical)
-    // screen estate
-    m_pLabel_Table->set_markup("<b>" + table_label + "</b>");
+  //Show the system's human-readable title and the table title in 
+  //the window's title bar:
+  Gtk::Window* app = get_app_window();
+  if(app)
+    app->set_title(document->get_name() + ": " + table_label);
+
+  // xx-large is too large on maemo, using far too much (vertical) space.
+  // We hide this anyway: m_pLabel_Table->set_markup("<b>" + table_label + "</b>");
 #else
-    m_pLabel_Table->set_markup("<b><span size=\"xx-large\">" + table_label + "</span></b>"); //Show the table title in large text, because it's very important to the user.
+  m_pLabel_Table->set_markup("<b><span size=\"xx-large\">" + table_label + "</span></b>"); //Show the table title in large text, because it's very important to the user.
 #endif
-  }
 }
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -1747,7 +1887,7 @@ void Frame_Glom::on_menu_developer_reports()
   if(!m_pBox_Reports)
   {
     Utils::get_glade_developer_widget_derived_with_warning("box_reports", m_pBox_Reports);
-    m_pDialog_Reports = new Dialog_Glom(m_pBox_Reports);
+    m_pDialog_Reports = new Window_BoxHolder(m_pBox_Reports);
     m_pDialog_Reports->set_transient_for(*(get_app_window()));
 
     Utils::get_glade_developer_widget_derived_with_warning("window_report_layout", m_pDialogLayoutReport);
@@ -1755,7 +1895,6 @@ void Frame_Glom::on_menu_developer_reports()
     m_pDialogLayoutReport->set_transient_for(*(get_app_window()));
     m_pDialogLayoutReport->signal_hide().connect( sigc::mem_fun(*this, &Frame_Glom::on_dialog_layout_report_hide) );
 
-    m_pDialog_Reports->get_vbox()->pack_start(*m_pBox_Reports);
     m_pDialog_Reports->set_default_size(300, 400);
     m_pBox_Reports->show_all();
 
@@ -1780,9 +1919,8 @@ void Frame_Glom::on_menu_developer_print_layouts()
   if(!m_pBox_PrintLayouts)
   {
     Utils::get_glade_developer_widget_derived_with_warning("box_print_layouts", m_pBox_PrintLayouts);
-    m_pDialog_PrintLayouts = new Dialog_Glom(m_pBox_PrintLayouts);
+    m_pDialog_PrintLayouts = new Window_BoxHolder(m_pBox_PrintLayouts);
 
-    m_pDialog_PrintLayouts->get_vbox()->pack_start(*m_pBox_PrintLayouts);
     m_pDialog_PrintLayouts->set_default_size(300, 400);
     m_pBox_PrintLayouts->show_all();
     add_view(m_pBox_PrintLayouts);
@@ -1900,6 +2038,8 @@ namespace
 #endif // GLOM_ENABLE_SQLITE
 
     default:
+      //on_document_load() should have checked for this already, informing the user.
+      std::cerr << "Glom: setup_connection_pool_from_document(): Unhandled hosting mode: " << document->get_hosting_mode() << std::endl;
       g_assert_not_reached();
       break;
     }
@@ -1912,6 +2052,7 @@ namespace
   }
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
 void Frame_Glom::on_connection_initialize_progress()
 {
   if(!m_dialog_progess_connection_initialize)
@@ -1919,7 +2060,7 @@ void Frame_Glom::on_connection_initialize_progress()
         
   m_dialog_progess_connection_initialize->pulse();
 }
-
+#endif //GLOM_ENABLE_CLIENT_ONLY
 
 void Frame_Glom::on_connection_startup_progress()
 {
@@ -1981,13 +2122,13 @@ bool Frame_Glom::connection_request_initial_password(Glib::ustring& user, Glib::
     return false;
 
   //Ask for a new username and password to specify when creating a new self-hosted database.
-  Dialog_NewSelfHostedConnection* dialog = 0;
+  Dialog_InitialPassword* dialog = 0;
   Glib::RefPtr<Gtk::Builder> refXml;
 
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-    refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_new_self_hosted_connection");
+    refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_initial_password");
   }
   catch(const Gtk::BuilderError& ex)
   {
@@ -1996,7 +2137,7 @@ bool Frame_Glom::connection_request_initial_password(Glib::ustring& user, Glib::
   }
 #else
   std::auto_ptr<Gtk::BuilderError> error;
-  refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_new_self_hosted_connection", error);
+  refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_initial_password", error);
   if(error.get())
   {
     std::cerr << error->what() << std::endl;
@@ -2004,7 +2145,7 @@ bool Frame_Glom::connection_request_initial_password(Glib::ustring& user, Glib::
   }
 #endif //GLIBMM_EXCEPTIONS_ENABLED
 
-  refXml->get_widget_derived("dialog_new_self_hosted_connection", dialog);
+  refXml->get_widget_derived("dialog_initial_password", dialog);
   if(!dialog)
     return false;
 
@@ -2105,7 +2246,7 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
 
     break;
 
-#ifdef GLOM_ENABLE_POSTGRES
+#ifdef GLOM_ENABLE_POSTGRESQL
     case Document::HOSTING_MODE_POSTGRES_CENTRAL:
     {
       //Ask for connection details:
@@ -2122,6 +2263,12 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
         ConnectionPool::SlotProgress slot_ignored;
         if(!handle_connection_initialize_errors( connection_pool->initialize(slot_ignored)) )
           return false;
+
+        // Remember the user name in the document, to be able to open the
+        // document again later:
+        Glib::ustring username, password;
+        m_pDialogConnection->get_username_and_password(username, password);
+        document->set_connection_user(username);
       }
       else
       {
@@ -2172,7 +2319,15 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
   {
     Glib::ustring database_name_possible;
     if(extra_num == 0)
-      database_name_possible = database_name; //Try the original name first.
+    {
+      //Try the original name first,
+      //removing any characters that are likely to cause problems when used in a SQL identifier name:
+      database_name_possible = Utils::trim_whitespace(database_name);
+      database_name_possible = Utils::string_replace(database_name_possible, "\"", "");
+      database_name_possible = Utils::string_replace(database_name_possible, "'", "");
+      database_name_possible = Utils::string_replace(database_name_possible, "\t", "");
+      database_name_possible = Utils::string_replace(database_name_possible, "\n", "");
+    }
     else
     {
       //Create a new database name by appending a number to the original name:
@@ -2187,14 +2342,16 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
     {
+      g_assert(m_pDialogConnection);
       sharedptr<SharedConnection> sharedconnection = m_pDialogConnection->connect_to_server_with_connection_settings();
       //If no exception was thrown then the database exists.
       //But we are looking for an unused database name, so we will try again.
     }
     catch(const ExceptionConnection& ex)
     {
-#else
+#else //GLIBMM_EXCEPTIONS_ENABLED
     std::auto_ptr<ExceptionConnection> error;
+    g_assert(m_pDialogConnection);
     sharedptr<SharedConnection> sharedconnection = m_pDialogConnection->connect_to_server_with_connection_settings(error);
     if(error.get())
     {
@@ -2214,18 +2371,20 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
         //The connection to the server is OK, but the specified database does not exist.
         //That's good - we were looking for an unused database name.
 
-        std::cout << "debug: unused database name found: " << database_name_possible << std::endl;
+        //std::cout << "debug: unused database name found: " << database_name_possible << std::endl;
         document->set_connection_database(database_name_possible);
 
-        // Remember host if the document is not self hosted
+        // Remember host and port if the document is not self hosted
         #ifdef GLOM_ENABLE_POSTGRESQL
         if(document->get_hosting_mode() == Document::HOSTING_MODE_POSTGRES_CENTRAL)
         {
           ConnectionPool::Backend* backend = connection_pool->get_backend();
           ConnectionPoolBackends::PostgresCentralHosted* central = dynamic_cast<ConnectionPoolBackends::PostgresCentralHosted*>(backend);
-          g_assert(central != NULL);
+          g_assert(central);
 
           document->set_connection_server(central->get_host());
+          document->set_connection_port(central->get_port());
+          document->set_connection_try_other_ports(false);
         }
 
         // Remember port if the document is self-hosted, so that remote
@@ -2238,9 +2397,10 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
         {
           ConnectionPool::Backend* backend = connection_pool->get_backend();
           ConnectionPoolBackends::PostgresSelfHosted* self = dynamic_cast<ConnectionPoolBackends::PostgresSelfHosted*>(backend);
-          g_assert(self != NULL);
+          g_assert(self);
 
           document->set_connection_port(self->get_port());
+          document->set_connection_try_other_ports(false);
         }
 
         #endif //GLOM_ENABLE_POSTGRESQL
@@ -2268,6 +2428,36 @@ void Frame_Glom::cleanup_connection()
   }
 }
 
+bool Frame_Glom::handle_request_password_connection_error(bool asked_for_password, const ExceptionConnection& ex, bool& database_not_found)
+{
+  g_warning("Frame_Glom::connection_request_password_and_attempt(): caught exception.");
+
+  //Initialize input parameter:
+  database_not_found = false;
+
+  if(asked_for_password && ex.get_failure_type() == ExceptionConnection::FAILURE_NO_SERVER)
+  {
+    //Warn the user, and let him try again:
+    Utils::show_ok_dialog(_("Connection Failed"), _("Glom could not connect to the database server. Maybe you entered an incorrect user name or password, or maybe the postgres database server is not running."), *(get_app_window()), Gtk::MESSAGE_ERROR); //TODO: Add help button.
+    return true;
+  }
+  else if(ex.get_failure_type() == ExceptionConnection::FAILURE_NO_DATABASE)
+  {
+    cleanup_connection();
+
+    //The connection to the server might be OK, but the specified database does not exist:
+    //Or the connection failed when trying without a password.
+    database_not_found = true; //Tell the caller about this error.
+    return false;
+  }
+  else
+  {
+    std::cerr << "Frame_Glom::connection_request_password_and_attempt(): Unexpected exception: " << ex.what() << std::endl;
+    cleanup_connection();
+    return false;
+  }
+}
+
 bool Frame_Glom::connection_request_password_and_attempt(bool& database_not_found, const Glib::ustring known_username, const Glib::ustring& known_password, bool confirm_known_user)
 {
   //Initialize output parameter:
@@ -2290,10 +2480,10 @@ bool Frame_Glom::connection_request_password_and_attempt(bool& database_not_foun
     m_dialog_progess_connection_startup = 0;
   }
 
-  //Only ask for the password if we are shared on the network.
+  //Only ask for the password if we are shared on the network, or we are using a centrally hosted server.
   //Otherwise, no password question is necessary, due to how our self-hosted database server is configured.
   if(document->get_network_shared()
-    && (document->get_hosting_mode() != Document::HOSTING_MODE_SQLITE) ) //TODO: The sqlite check may be unnecessary.
+    || document->get_hosting_mode() == Document::HOSTING_MODE_POSTGRES_CENTRAL)
   {
     //We recreate the dialog each time to make sure it is clean of any changes:
     if(m_pDialogConnection)
@@ -2332,7 +2522,7 @@ bool Frame_Glom::connection_request_password_and_attempt(bool& database_not_foun
     //Only show the dialog if we don't know the correct username/password yet:
     int response = Gtk::RESPONSE_OK;
 
-    if(m_pDialogConnection && known_username.empty() && known_password.empty())
+    if(m_pDialogConnection)
     {
       response = Glom::Utils::dialog_run_with_help(m_pDialogConnection, "dialog_connection");
       m_pDialogConnection->hide();
@@ -2341,70 +2531,63 @@ bool Frame_Glom::connection_request_password_and_attempt(bool& database_not_foun
     //Try to use the entered username/password:
     if(response == Gtk::RESPONSE_OK)
     {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-      try
+      sharedptr<SharedConnection> sharedconnection;
+
+      //Ask for the user/password if necessary:
+      //TODO: Remove any previous database setting?
+      if(m_pDialogConnection)
       {
-        //TODO: Remove any previous database setting?
-        if(m_pDialogConnection)
+        #ifdef GLIBMM_EXCEPTIONS_ENABLED
+        try
         {
-          sharedptr<SharedConnection> sharedconnection = m_pDialogConnection->connect_to_server_with_connection_settings();
+          sharedconnection = m_pDialogConnection->connect_to_server_with_connection_settings();
+          // TODO: Save username in document?
           return true; //Succeeded, because no exception was thrown.
         }
-        else
+        catch(const ExceptionConnection& ex)
         {
-          //Use the known password:
-          ConnectionPool* connectionpool = ConnectionPool::get_instance();
-          connectionpool->set_user(known_username);
-          connectionpool->set_password(known_password);
-    
-          #ifdef GLIBMM_EXCEPTIONS_ENABLED
-          Base_DB::connect_to_server(get_app_window());
-          return true; //Succeeded, because no exception was thrown.
-          #else
-          std::auto_ptr<ExceptionConnection> error;
-          const bool connected = Base_DB::connect_to_server(get_app_window(), error);
-          if(!connected || error)
+          if(!handle_request_password_connection_error(true, ex, database_not_found))
             return false;
-          else
-            return true;
-          #endif
         }
+        #else //GLIBMM_EXCEPTIONS_ENABLED
+        std::auto_ptr<ExceptionConnection> local_error;
+        sharedconnection = 
+          m_pDialogConnection->connect_to_server_with_connection_settings(local_error);
+        if(!local_error.get())
+          return true;
+        else if(!handle_request_password_connection_error(true, *local_error, database_not_found))
+          return false;
+        #endif //GLIBMM_EXCEPTIONS_ENABLED
       }
-      catch(const ExceptionConnection& ex)
-      {
-#else
-      std::auto_ptr<ExceptionConnection> local_error;
-      sharedptr<SharedConnection> sharedconnection = m_pDialogConnection->connect_to_server_with_connection_settings(local_error);
-      if(!local_error.get())
-        return true;
       else
       {
-        const ExceptionConnection& ex = *local_error;
-#endif
-        g_warning("Frame_Glom::connection_request_password_and_attempt(): caught exception.");
-
-        if(m_pDialogConnection && ex.get_failure_type() == ExceptionConnection::FAILURE_NO_SERVER)
+        //Use the known password:
+        ConnectionPool* connectionpool = ConnectionPool::get_instance();
+        connectionpool->set_user(known_username);
+        connectionpool->set_password(known_password);
+    
+        #ifdef GLIBMM_EXCEPTIONS_ENABLED
+        try
         {
-          //Warn the user, and let him try again:
-          Utils::show_ok_dialog(_("Connection Failed"), _("Glom could not connect to the database server. Maybe you entered an incorrect user name or password, or maybe the postgres database server is not running."), *(get_app_window()), Gtk::MESSAGE_ERROR); //TODO: Add help button.
-         
-          //The while() loop will run again, showing the username/password dialog again.
+          Base_DB::connect_to_server(get_app_window());
+          return true; //Succeeded, because no exception was thrown.
         }
-        else if (ex.get_failure_type() == ExceptionConnection::FAILURE_NO_DATABASE)
+        catch(const ExceptionConnection& ex)
         {
-          cleanup_connection();
-
-          //The connection to the server might be OK, but the specified database does not exist:
-          //Or the connection failed when trying without a password.
-          database_not_found = true; //Tell the caller about this error.
-          return false;
+          if(!handle_request_password_connection_error(false, ex, database_not_found))
+            return false;
+        }
+        #else
+        std::auto_ptr<ExceptionConnection> error;
+        const bool connected = Base_DB::connect_to_server(get_app_window(), error);
+        if(!connected || error.get())
+        {
+          if(!handle_request_password_connection_error(false, *error, database_not_found))
+            return false;
         }
         else
-        {
-          std::cerr << "Frame_Glom::connection_request_password_and_attempt(): Unexpected exception: " << ex.what() << std::endl;
-          cleanup_connection();
-          return false;
-        }
+          return true;
+        #endif
       }
 
       //Try again.
@@ -2421,7 +2604,7 @@ bool Frame_Glom::connection_request_password_and_attempt(bool& database_not_foun
 bool Frame_Glom::create_database(const Glib::ustring& database_name, const Glib::ustring& title)
 {
 #if 1
-  // This seems to increase the change that the database creation does not
+  // This seems to increase the chance that the database creation does not
   // fail due to the "source database is still in use" error. armin.
   //std::cout << "Going to sleep" << std::endl;
   Glib::usleep(500 * 1000);
@@ -2576,7 +2759,7 @@ void Frame_Glom::on_menu_print_layout_selected(const Glib::ustring& print_layout
     Glib::KeyFile key_file;
     key_file.load_from_data(key_file_text);
     //TODO: Use this when gtkmm and GTK+ have been fixed: page_setup = Gtk::PageSetup::create(key_file);
-    page_setup = Glib::wrap(gtk_page_setup_new_from_key_file(key_file.gobj(), NULL, NULL));
+    page_setup = Glib::wrap(gtk_page_setup_new_from_key_file(key_file.gobj(), 0, 0));
   }
   
   print->set_default_page_setup(page_setup);
@@ -2758,5 +2941,10 @@ bool Frame_Glom::get_viewing_details() const
   return (m_Notebook_Data.get_current_view() == Notebook_Data::DATA_VIEW_Details);
 }
 
+Glib::ustring Frame_Glom::get_shown_table_name() const
+{
+  return m_table_name;
+}
+
 } //namespace Glom
 
