@@ -23,26 +23,41 @@
 
 #include <gtkmm.h>
 #include "layoutwidgetbase.h"
-#include "../data_structure/layout/layoutitem_button.h"
-#include <libglademm.h>
+#include "layoutwidgetutils.h"
+#include <libglom/data_structure/layout/layoutitem_button.h>
+#include <gtkmm/builder.h>
+
+namespace Glom
+{
 
 class App_Glom;
 
 class LabelGlom
-: public Gtk::Label,
-  public LayoutWidgetBase
+: public Gtk::EventBox,
+  public LayoutWidgetUtils
 {
 public:
-  explicit LabelGlom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
-  explicit LabelGlom();
+  LabelGlom();
+  explicit LabelGlom(const Glib::ustring& label, bool mnemonic = false);
   explicit LabelGlom(const Glib::ustring& label, float xalign, float yalign, bool mnemonic = false);
   virtual ~LabelGlom();
+  
+  Gtk::Label* get_label();
 
-protected:
+private:
   void init();
 
   virtual App_Glom* get_application();
+    
+  Gtk::Label m_label;
+#ifndef GLOM_ENABLE_CLIENT_ONLY    
+  virtual bool on_button_press_event(GdkEventButton *event);
+  virtual void on_menu_properties_activate();
+#endif // !GLOM_ENABLE_CLIENT_ONLY
+    
 };
 
+} //namespace Glom
+
 #endif //GLOM_UTILITY_WIDGETS_LABEL_GLOM_H
 
